/*--------------------------------------------------------------------*\
  File:      dev_server.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1998, Matt Bogosian. All rights reserved.
  Description: Header file describing the dev_server protocol.
  ID:        $Id: dev_server.h,v 1.16 1998/07/08 05:36:02 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include <OS.h>


#ifndef DEV_SERVER_H
#define DEV_SERVER_H

#ifdef __cplusplus
extern "C"
{
#endif    // __cplusplus


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

typedef status_t (*NewAddOnFunc)(const void * const, class AddOn ** const);

// The name of the port on which the dev_server listens for and
// accepts connections
extern const char k_dvsrvr_accpt_name[B_OS_NAME_LENGTH];

// The maximum message length for the dev_server connection request
extern const size_t k_dvsrvr_max_msg_len;

// The message code for the dev_server connection request
extern const int32 k_dvsrvr_add_on_create;

// The message code for the dev_server add-on destruction request
extern const int32 k_dvsrvr_add_on_destroy;


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=- Function Prototypes =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

/*--------------------------------------------------------------------*\
  Function name: dvsrvrConnect
  Defined in:    dev_server.cpp
  Arguments:     const char a_name[B_FILE_NAME_LENGTH] - the name of
                     the corresponding dev_server addon to load.
                 const port_id a_recv - the port on which messages
                     will be received.
                 const port_id a_send - the port on which messages
                     will be sent (this could be the same as the
                     receiving port in certain cases). Default:
                     B_BAD_PORT_ID.
  Returns:       status_t - the error code associated with the
                     operation (see below).
  Throws:        none
  Description: Function to attempt to make a connection to the
      dev_server. Possible return values are as follows:
      
      B_NO_ERROR - the connection was created successfully.
      B_ERROR - the connection could not be made.
      B_NO_MEMORY - there was not enough memory to create the
          connection.
      B_BAD_PORT_ID - the dev_server accept port could not be found or
          the receiving port was no good.
      B_TIMED_OUT - the dev_server did not respond.
\*--------------------------------------------------------------------*/

status_t dvsrvrConnect(const char a_name[B_FILE_NAME_LENGTH], const port_id a_recv, const port_id a_send = B_BAD_PORT_ID);

/*--------------------------------------------------------------------*\
  Function name: inline dvsrvrKillAddOn
  Arguments:     AddOn * const a_add_on - the add-on of which to
                     request the destruction.
  Returns:       status_t - the error code associated with the
                     operation (see below).
  Throws:        none
  Description: Function to attempt to make a connection to the
      dev_server and request the destruction of an add-on. Possible
      return values are as follows:
      
      B_NO_ERROR - the request was sent successfully.
      B_BAD_PORT_ID - the dev_server accept port could not be found.
\*--------------------------------------------------------------------*/

inline status_t dvsrvrKillAddOn(AddOn * const a_add_on)
{
	return write_port(find_port(k_dvsrvr_accpt_name), k_dvsrvr_add_on_destroy, &a_add_on, sizeof (a_add_on));
}

/*--------------------------------------------------------------------*\
  Function name: extern dvsrvrNewAddon
  Arguments:     const void * const a_data - the necessary basic data
                     for the add-on given by the dev_server. You must
                     pass this straight to this constructor for the
                     add-on. Do not pass go. Do not collect $200. (Bad
                     things may happen if the add-on is robbed of this
                     data.)
                 AddOn ** const a_add_on - a placeholder for the
                     newly-created dev_server add-on. Don't put
                     anything in this placeholder unless you return
                     success (see below).
  Returns:       status_t - the error code associated with the
                     operation (see below).
  Throws:        none
  Description: Entry function defined by dev_server add-ons to
      create a connection with the dev_server. Connections between
      drivers and dev_server add-ons are made via a two-step process.
      The calling of this function is the first step. If the call to
      this function is successful (as indicated by a return code of
      B_NO_ERROR), then the second step is made by calling the Start()
      member function of the add-on returned by this function.
      
      This function should create a new dev_server add-on and return
      it to the caller via the a_add_on placeholder. You should
      probably wait to spawn your own threads until the returned add-
      on's Start() member function is called.
      
      The function should return B_NO_ERROR on success, or an
      appropriate error code on failure. This error code will be sent
      back to the driver. If the function returns anything other than
      B_NO_ERROR, then the dev_server will make no efforts to call the
      Start() method of whatever ends up in a_add_on.
      
      Note: this function has been extern "C"'d, and should be an
      exported symbol in the dev_server add-on. In other words, a
      dev_server add-on's declaration should look something like what
      follows:
      
        #include "dev_server.h"
        
        #pragma export on
        
        extern "C" __declspec(dllexport) status_t dvsrvrNewAddon
          (const void * const a_data, AddOn ** const a_add_on)
        {
          // ...
        }
        
        #pragma export off
      
      Make sure to create a new add-on and give it to the caller via
      the a_add_on placeholder. Bad things happen if this is not done.
\*--------------------------------------------------------------------*/

extern status_t dvsrvrNewAddon(const void * const a_data, const port_id a_send, const port_id a_recv, AddOn ** const a_add_on);


#ifdef __cplusplus
}
#endif    // __cplusplus

#endif    // DEV_SERVER_H
